var md = require('../kaverti_modules/markdown-it')({
    html:         true,        // Enable HTML tags in source
    xhtmlOut:     false,        // Use '/' to close single tags (<br />).
                                // This is only for full CommonMark compatibility.
    breaks:       true,        // Convert '\n' in paragraphs into <br>
    langPrefix:   'language-',  // CSS language prefix for fenced blocks. Can be
                                // useful for external highlighters.
    linkify:      true,        // Autoconvert URL-like text to links
    image: true,

    // Enable some language-neutral replacement + quotes beautification
    typographer:  true,

    // Double + single quotes replacement pairs, when typographer enabled,
    // and smartquotes on. Could be either a String or an Array.
    //
    // For example, you can use '«»„“' for Russian, '„“‚‘' for German,
    // and ['«\xA0', '\xA0»', '‹\xA0', '\xA0›'] for French (including nbsp).
    quotes: '“”‘’',

    // Highlighter function. Should return escaped HTML,
    // or '' if the source string is not changed and should be escaped externally.
    // If result starts with <pre... internal wrapper is skipped.
    highlight: function (/*str, lang*/) { return ''; }
});
var emoji = require('markdown-it-emoji');
var twemoji = require('twemoji')
md.use(emoji);
md.renderer.rules.emoji = function(token, idx) {
    return twemoji.parse(token[idx].content);
};
module.exports = (sequelize, DataTypes) => {
    let BlogPost = sequelize.define('BlogPost', {
        name: {
            type: DataTypes.TEXT,
            set (val) {
                this.setDataValue('name', val)
                if(val) {
                    this.setDataValue(
                        'slug',
                        //if you don't covert to lowercase it doesn't
                        //correctly slugify diacritics, e.g. thrËad
                        //becomes 'thr-ead' not 'thread'
                        urlSlug(val.toString().toLowerCase() || '') || '_'
                    )
                }
            },
            allowNull: false,
            validate: {
                notEmpty: {
                    msg: 'Did you forget something? (Fill this in!)'
                },
                len: {
                    args: [4, 256],
                    msg: 'The title must be between 4 and 256 characters'
                },
                isString (val) {
                    if(typeof val !== 'string') {
                        throw new sequelize.ValidationError('The title must be a string')
                    }
                }
            }
        },
        slug: {
            type: DataTypes.TEXT,
            defaultValue: "deprecated"
        },
        plainText: {
            type: DataTypes.TEXT,
        },
        content: {
          type: DataTypes.TEXT
        },
        createdAt: DataTypes.DATE,
        updatedAt: DataTypes.DATE,
    })
    return BlogPost
}
