let validation = require('../lib/validation/validateMiddleware');
let conversationController = require('../controllers/conversation');
let router = require('express').Router();
const auth = require('../lib/auth')

let createPostSchema = {
	body: {
		userIds: {
			required: true,
			type: 'array.integer'
		},
		name: {
			required: false,
			type: 'string'
		}
	}
};
router.post('/', auth, validation(createPostSchema), async (req, res, next) => {
	try {
		let conversation = await conversationController.create(
			req.body.userIds,
			req.body.name
		);

		res.json(conversation);
	} catch (e) { next(e); }
});

let getPostSchema = {
	params: {
		conversationId: {
			required: true,
			type: 'string(integer)'
		}
	},
	query: {
		page: {
			required: false,
			type: 'string(integer)'
		}
	}
};
router.get('/:conversationId', auth, validation(getPostSchema), async (req, res, next) => {
	try {
		let conversation = await conversationController.get(
			req.userData.id, +req.params.conversationId, +req.query.page
		);

		res.json(conversation);
	} catch (e) { next(e); }
});

let putPostSchema = {
	params: {
		conversationId: {
			required: true,
			type: 'string(integer)'
		}
	}
};
router.put('/:conversationId', auth, validation(putPostSchema), async (req, res, next) => {
	try {
		let conversationId = +req.params.conversationId;
		res.json(
			await conversationController.updateLastRead(conversationId, req.userData.id)
		);
	} catch (e) { next(e); }
});

let putNameSchema = {
	params: {
		conversationId: {
			required: true,
			type: 'string(integer)'
		}
	},
	body: {
		name: {
			required: true,
			type: 'string'
		}
	}
};
router.put('/:conversationId/name', auth, validation(putNameSchema), async (req, res, next) => {
	try {
		let conversationId = +req.params.conversationId;
		let name = req.body.name;

		res.json(
			await conversationController.updateName(conversationId, req.userData.id, name)
		);
	} catch (e) { next(e); }
});

module.exports = router;